/* -----------------------------------------------------------------------------
 * Copyright (c) 2021 ARM Ltd.
 *
 * This software is provided 'as-is', without any express or implied warranty.
 * In no event will the authors be held liable for any damages arising from
 * the use of this software. Permission is granted to anyone to use this
 * software for any purpose, including commercial applications, and to alter
 * it and redistribute it freely, subject to the following restrictions:
 *
 * 1. The origin of this software must not be misrepresented; you must not
 *    claim that you wrote the original software. If you use this software in
 *    a product, an acknowledgment in the product documentation would be
 *    appreciated but is not required.
 *
 * 2. Altered source versions must be plainly marked as such, and must not be
 *    misrepresented as being the original software.
 *
 * 3. This notice may not be removed or altered from any source distribution.
 *
 *
 * $Date:        17. July 2021
 * $Revision:    V1.0.0
 *
 * Project:      Flash Programming Functions for ST STM32U5xx Flash
 * --------------------------------------------------------------------------- */

/* History:
 *  Version 1.0.0
 *    Initial release
 */

/* Note:
   Flash has 8K sector size.
   STM32U5xx devices have Dual/Single Bank Flash configuration .  */

#include "..\FlashOS.h"        // FlashOS Structures

typedef volatile unsigned long    vu32;
typedef          unsigned long     u32;
typedef volatile unsigned long    vu64;
typedef          unsigned long     u64;



#define M32(adr) (*((vu32 *) (adr)))
#define M64(adr) (*((vu64 *) (adr)))

// Peripheral Memory Map
#define FLASH_BASE        (0x40022000)
#define DBGMCU_BASE       (0xE0044000)

#define FLASH           ((FLASH_TypeDef  *) FLASH_BASE)
#define DBGMCU          ((DBGMCU_TypeDef *) DBGMCU_BASE)
#define FLASHSIZE_BASE    (0x0BFA05E0)

// Debug MCU
typedef struct {
  vu32 IDCODE;
} DBGMCU_TypeDef;

// Flash Registers
typedef struct {
    vu32 ACR;                /*!< FLASH access control register,              Address offset: 0x00 */
   vu32 PDKEYR;             /*!< FLASH power down key register,              Address offset: 0x04 */
   vu32 NSKEYR;             /*!< FLASH non secure key register,              Address offset: 0x08 */
   vu32 SECKEYR;            /*!< FLASH  secure key register,                Address offset: 0x0C */
   vu32 OPTKEYR;            /*!< FLASH option key register,                  Address offset: 0x10 */
   vu32 LVEKEYR;            /*!< FLASH  LVE key register,                    Address offset: 0x14 */
   vu32 RESERVED01;          /*!< Reserved,                                 Address offset: 0x18 */
   vu32 RESERVED02;          /*!< Reserved,                                 Address offset: 0x1C */
   vu32 NSSR;               /*!< FLASH non secure status register,           Address offset: 0x20 */
   vu32 SECSR;              /*!< FLASH secure status register,              Address offset: 0x24 */
   vu32 NSCR;               /*!< FLASH non secure control register,         Address offset: 0x28 */
   vu32 SECCR;              /*!< FLASH  secure control register,             Address offset: 0x2C */
   vu32 RESERVED30;          /*!< Reserved,                                   Address offset: 0x30 */
   vu32 RESERVED34;          /*!< Reserved,                                   Address offset: 0x34 */
   vu32 RESERVED38;          /*!< Reserved,                                   Address offset: 0x38 */
   vu32 RESERVED3C;          /*!< Reserved,                                   Address offset: 0x3C */
   vu32 OPTR;                /*!< FLASH option control register,             Address offset: 0x40 */
   vu32 RESERVED44;          /*!< Reserved,                                   Address offset: 0x44 */
   vu32 RESERVED48;          /*!< Reserved,                                   Address offset: 0x48 */
   vu32 RESERVED4C;          /*!< Reserved,                                   Address offset: 0x4C */
   vu32 RESERVED50;          /*!< Reserved,                                   Address offset: 0x50 */
   vu32 RESERVED54;          /*!< Reserved,                                   Address offset: 0x54 */
   vu32 RESERVED58;          /*!< Reserved,                                   Address offset: 0x58 */
   vu32 RESERVED5C;          /*!< Reserved,                                   Address offset: 0x5C */
   vu32 RESERVED60;          /*!< Reserved,                                   Address offset: 0x60 */
   vu32 RESERVED64;          /*!< Reserved,                                   Address offset: 0x64 */
   vu32 RESERVED68;          /*!< Reserved,                                   Address offset: 0x68 */
   vu32 RESERVED6C;          /*!< Reserved,                                   Address offset: 0x6C */
   vu32 RESERVED70;          /*!< Reserved,                                   Address offset: 0x70 */
   vu32 RESERVED74;          /*!< Reserved,                                   Address offset: 0x74 */
   vu32 RESERVED78;          /*!< Reserved,                                   Address offset: 0x78 */
   vu32 RESERVED7C;          /*!< Reserved,                                   Address offset: 0x7C */
   vu32 SECBBA0;            /*!< FLASH block-based secure A,                Address offset: 0x80 */
   vu32 SECBBA1;            /*!< FLASH block-based secure A,                Address offset: 0x84 */
   vu32 SECBBA2;            /*!< FLASH block-based secure A,                Address offset: 0x88 */
   vu32 SECBBA3;            /*!< FLASH block-based secure A,                Address offset: 0x8C */
   vu32 RESERVED90;         /*!< Reserved,                                  Address offset: 0x90 */
   vu32 RESERVED94;         /*!< Reserved,                                  Address offset: 0x94 */
   vu32 RESERVED98;         /*!< Reserved,                                  Address offset: 0x98 */
   vu32 RESERVED9C;         /*!< Reserved,                                  Address offset: 0x9C */
   vu32 SECBBB0;            /*!< FLASH block-based secure B,                Address offset: 0xA0 */
   vu32 SECBBB1;            /*!< FLASH block-based secure B,                Address offset: 0xA4 */
   vu32 SECBBB2;            /*!< FLASH block-based secure B,                Address offset: 0xA8 */
   vu32 SECBBB3;            /*!< FLASH block-based secure B,                Address offset: 0xAC */
} FLASH_TypeDef;

// Flash Keys
#define FLASH_KEY1               0x45670123
#define FLASH_KEY2               0xCDEF89AB
#define FLASH_OPTKEY1            0x08192A3B
#define FLASH_OPTKEY2            0x4C5D6E7F

#define FLASH_PG                ((u32)(1U <<  0))
#define FLASH_PER               ((u32)(1U <<  1))
#define FLASH_MER1              ((u32)(1U <<  2))
#define FLASH_PNB_MSK           ((u32)(0xFF << 3))
#define FLASH_BKER              ((u32)(1U << 11))
#define FLASH_MER2              ((u32)(1U << 15))
#define FLASH_STRT              ((u32)(1U << 16))
#define FLASH_LOCK              ((u32)(1U << 31))


// Flash Status Register definitions
#define FLASH_EOP               ((u32)(1U <<  0))
#define FLASH_OPERR             ((u32)(1U <<  1))
#define FLASH_PROGERR           ((u32)(1U <<  3))
#define FLASH_WRPERR            ((u32)(1U <<  4))
#define FLASH_PGAERR            ((u32)(1U <<  5))
#define FLASH_SIZERR            ((u32)(1U <<  6))
#define FLASH_PGSERR            ((u32)(1U <<  7))
#define FLASH_MISSERR           ((u32)(1U <<  8))
#define FLASH_FASTERR           ((u32)(1U <<  9))
#define FLASH_RDERR             ((u32)(1U << 14))

#define FLASH_SECSR_SECBSY       ((u32)0x00010000)
#define FLASH_SECSR_SECOPERR     ((u32)0x00000002)
#define FLASH_SECCR_SECPG        ((u32)0x00000001)
#define FLASH_SECSR_SECEOP       ((u32)0x00000001)
#define FLASH_SECCR_SECPER       ((u32)0x00000002)
#define FLASH_SECCR_SECPNB       ((u32)0x000007F8)
#define FLASH_SECCR_SECSTRT      ((u32)0x00010000)
#define FLASH_SECCR_SECLOCK      ((u32)0x80000000)
#define FLASH_SECCR_NSMERA       ((u32)0x00000004)
#define FLASH_SECCR_NSMERB       ((u32)0x00008000)

#define FLASH_NSSR_NSWDW         ((u32)0x00020000)

#define FLASH_PGERR             (FLASH_OPERR  | FLASH_PROGERR | FLASH_PROGERR | FLASH_WRPERR  | FLASH_PGAERR | \
                                 FLASH_SIZERR | FLASH_PGSERR  | FLASH_MISSERR | FLASH_FASTERR | FLASH_RDERR   )


// Flash option register definitions
#define FLASH_OPTR_RDP          ((u32)(0xFF ))
#define FLASH_OPTR_RDP_NO       ((u32)(0xAA  ))
#define FLASH_OPTR_DBANK        ((u32)( 1U << 21))
#define FLASH_OPTR_TZEN        ((u32)( 1U << 31))
#define FLASH_OBL_LAUNCH        ((u32)( 1U << 27))


u32 flashBase;                   /* Flash base address */
u32 flashSize;                   /* Flash size in bytes */
u32 flashBankSize;               /* Flash bank size in bytes */
u32 flashPageSize;               /* Flash page size in bytes */

static void DSB(void) {
    __asm("DSB");
}


/*
 * Get Flash Bank Mode
 *    Return Value:   0 = Single-Bank mode
 *                    1 = Dual-Bank mode
 */


static u32 GetFlashBankMode (void) {
  u32 flashBankMode;

  flashBankMode = (FLASH->OPTR & FLASH_OPTR_DBANK) ? 1U : 0U;

  return (flashBankMode);
}


/*
 * Get Flash security Mode
 *    Return Value:   0 = non-secure Flash
 *                    1 = secure Flash
 */

static u32 GetFlashSecureMode (void) {
  u32 flashSecureMode;

  flashSecureMode = (FLASH->OPTR & FLASH_OPTR_TZEN) ? 1U : 0U;

  return (flashSecureMode);
}

/*
 * Get Flash Page Size
 *    Return Value:   flash page size (in Bytes)
 */


static u32 GetFlashPageSize (void) {
  u32 flashPageSize;
if (GetFlashSecureMode() == 1U) {                  // Flash secure
    /* Dual-Bank Flash */
    if (GetFlashBankMode() == 1U) {
      /* Dual-Bank Flash configured as Dual-Bank */
      flashPageSize = 0x2000;                                /* 8K sector size */
    }
    else {
      /* Dual-Bank Flash configured as Single-Bank */
      flashPageSize = 0x4000;                                /* 16K sector size */
    }
  }
    else                                        // Flash non-secure
    {
          /* Dual-Bank Flash */
    if (GetFlashBankMode() == 1U) {
      /* Dual-Bank Flash configured as Dual-Bank */
      flashPageSize = 0x2000;                                /* 8K sector size */
    }
    else {
      /* Dual-Bank Flash configured as Single-Bank */
      flashPageSize = 0x4000;                                /* 16K sector size */
    }
  }
  return (flashPageSize);
}



/*
 *  Initialize Flash Programming Functions
 *    Parameter:      adr:  Device Base Address
 *                    clk:  Clock Frequency (Hz)
 *                    fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int Init (unsigned long adr, unsigned long clk, unsigned long fnc) {

  if (GetFlashSecureMode() == 0U)    // Flash non-secure
    {
    /* Unlock Flash */
    if ((FLASH->NSCR & FLASH_LOCK) != 0){
    FLASH->NSKEYR = FLASH_KEY1;
    FLASH->NSKEYR = FLASH_KEY2;
  }
    /*Wait until the flash is ready*/
    while (FLASH->NSSR & FLASH_SECSR_SECBSY);
    }
  else                                // Flash secure
    {
      if (GetFlashBankMode() == 1U)  // Flash secure DUAL BANK
      {
    /* Flash block-based secure */
    FLASH-> SECBBA0 = 0xffffffff;
    FLASH-> SECBBA1 = 0xffffffff;
    FLASH-> SECBBA2 = 0xffffffff;
    FLASH-> SECBBA3 = 0xffffffff;
    FLASH-> SECBBB0 = 0xffffffff;
    FLASH-> SECBBB1 = 0xffffffff;
    FLASH-> SECBBB2 = 0xffffffff;
    FLASH-> SECBBB3 = 0xffffffff;
    /* Unlock Flash*/
    FLASH->SECKEYR = FLASH_KEY1;
    FLASH->SECKEYR = FLASH_KEY2;
    /*Wait until the flash is ready*/
    while (FLASH->SECSR & FLASH_SECSR_SECBSY);
    }
      else                                 // Flash secure SINGLE BANK
       {
          /* Flash block-based secure */
          FLASH-> SECBBA0 = 0xffffffff;
          FLASH-> SECBBA1 = 0xffffffff;
          FLASH-> SECBBA2 = 0xffffffff;
          FLASH-> SECBBA3 = 0xffffffff;
          /* Unlock Flash*/
          FLASH->SECKEYR = FLASH_KEY1;
          FLASH->SECKEYR = FLASH_KEY2;
          /*Wait until the flash is ready*/
          while (FLASH->SECSR & FLASH_SECSR_SECBSY);

      }
  }


#ifdef FLASH_OPT
  FLASH->OPTKEYR  = FLASH_OPTKEY1;                       /* Unlock Option Bytes operation */
  FLASH->OPTKEYR  = FLASH_OPTKEY2;
#endif /* FLASH_OPT */

  if (((FLASH->OPTR & FLASH_OPTR_RDP) != FLASH_OPTR_RDP_NO))
  {
    FLASH->OPTKEYR = FLASH_OPTKEY1;
    FLASH->OPTKEYR = FLASH_OPTKEY2;

    /* clear  read protection */
    FLASH->OPTR &= ~(FLASH_OPTR_RDP);
    FLASH->OPTR |= FLASH_OPTR_RDP_NO;

    FLASH->NSCR |= FLASH_SECCR_SECSTRT;
    FLASH->NSCR |= FLASH_OBL_LAUNCH;
    DSB();

    /* Wait until option bytes are updated */
    while (FLASH->NSCR & FLASH_OBL_LAUNCH);
  }

  /* Wait until the flash is ready */
  while (FLASH->NSSR & FLASH_SECSR_SECBSY);

  flashBase = adr;
  flashSize = ((*((u32 *)FLASHSIZE_BASE)) & 0x0000FFFF) << 10;
  flashBankSize = flashSize >> 1;
  flashPageSize = GetFlashPageSize();

  return (0);
}

/*
 *  De-Initialize Flash Programming Functions
 *    Parameter:      fnc:  Function Code (1 - Erase, 2 - Program, 3 - Verify)
 *    Return Value:   0 - OK,  1 - Failed
 */

int UnInit (unsigned long fnc) {

   if (GetFlashSecureMode() == 0U)   // Flash non-secure
    {
      FLASH->NSCR |= FLASH_LOCK;                            /* Lock Flash operation */
    }
    else                                // Flash secure
    {
      /* lock Flash*/
       FLASH->SECCR |= FLASH_SECCR_SECLOCK;
    }
  DSB();

#ifdef FLASH_OPT
  FLASH->NSCR  = FLASH_OBL_LAUNCH;                         /* Load option bytes */
  DSB();
  while (FLASH->NSCR & FLASH_OBL_LAUNCH);

  FLASH->NSCR = FLASH_OPTLOCK;                             /* Lock option bytes operation */
  DSB();
#endif /* FLASH_OPT */

  return (0);
}

/*
 *  Erase complete Flash Memory
 *    Return Value:   0 - OK,  1 - Failed
 */


int EraseChip (void) {

    if (GetFlashSecureMode() == 0U)    // Flash non-secure
    {
     while (FLASH->SECSR & FLASH_SECSR_SECBSY){};
     FLASH->NSSR  = FLASH_PGERR;                              /* Reset Error Flags */

     FLASH->NSCR |= (FLASH_MER1 | FLASH_MER2);          /* Bank A/B mass rrase enabled */
     FLASH->NSCR |=  FLASH_STRT;                           /* Start erase */
     DSB();
  /*wait until the operation ends*/
    while (FLASH->NSSR & FLASH_SECSR_SECBSY);
      /*reset CR*/
    FLASH->NSCR &= (~FLASH_MER1);
    FLASH->NSCR &= (~FLASH_MER2);
    }
  else{                                // Flash secure

    while (FLASH->SECSR & FLASH_SECSR_SECBSY){};
    FLASH->SECSR  = FLASH_PGERR;                              /* Reset Error Flags */

    FLASH->SECCR  = (FLASH_MER1 | FLASH_MER2);          /* Bank A/B mass rrase enabled */
    FLASH->SECCR |=  FLASH_STRT;                           /* Start erase */
    DSB();

    while (FLASH->SECSR & FLASH_SECSR_SECBSY);
    /*reset CR*/
    FLASH->NSCR &= (~FLASH_MER1);
    FLASH->NSCR &= (~FLASH_MER2);
  }
  return (0);
}


#ifdef FLASH_OPT
int EraseChip (void) {

  FLASH->NSSR  = FLASH_PGERR;                              /* Reset Error Flags */

  FLASH->OPTR     = 0xFFEFF8AA;
  FLASH->PCROP1SR = 0x000000FF;
  FLASH->PCROP1ER = 0x00000000;
  FLASH->WRP1AR   = 0x000000FF;
  FLASH->WRP1BR   = 0x000000FF;
  FLASH->SEC1R    = 0x00000000;
  if (GetFlashType() == 1U) {
  FLASH->PCROP1SR = 0x000000FF;
  FLASH->PCROP1ER = 0x00000000;
  FLASH->WRP1AR   = 0x000000FF;
  FLASH->WRP1BR   = 0x000000FF;
  FLASH->SEC1R    = 0x00000000;
  }

  FLASH->NSCR       = FLASH_OPTSTRT;                       /* Program values */
  DSB();

  while (FLASH->NSSR & FLASH_SECSR_SECBSY);

  if (FLASH->NSSR & FLASH_PGERR) {                         /* Check for Error */
    FLASH->NSSR  = FLASH_PGERR;                            /* Reset Error Flags */
    return (1);                                          /* Failed */
  }

  return (0);                                            /* Done */
}
#endif /* FLASH_OPT */


/*
 *  Erase Sector in Flash Memory
 *    Parameter:      adr:  Sector Address
 *    Return Value:   0 - OK,  1 - Failed
 */


int EraseSector (unsigned long adr) {
  unsigned long page;
u32 bankADD;
  if (adr >= 0x0C000000)
  {
    adr = adr - 0x04000000;
  }

  #if defined( STM32U5xx_1024_0x0C) || defined(STM32U5xx_1024)
  bankADD=0x08080000;
  if (GetFlashSecureMode() == 0U) {   // Flash non-secure
    {
      if (GetFlashBankMode() == 1U)  // Flash non-secure DUAL BANK
      {
    /*page erase enabled*/
    FLASH->NSCR |= FLASH_SECCR_SECPER;

    if(adr >= bankADD) // bank 2
        {
        page = (adr >>13);
        page = page - 0x80;
        page = page & 0x0000007F;
        /*Bank 2 Erase Selection*/
        FLASH->NSCR |= (FLASH_BKER);
        }
    else               // bank 1
        {
         page = ((adr >>13) & 0x0000007F );
        /*Bank 1 Erase Selection*/
         FLASH->NSCR &= ~(FLASH_BKER);
        }
      }
      else{                      // Flash secure SINGLE BANK
        /*Calculate the page */
              page = ((adr >>14) & 0x000000FF );
      }
    /*set PNB*/
    FLASH->NSCR &= ~(FLASH_SECCR_SECPNB);
    FLASH->NSCR |= page << 3;

    /*Start erase operation*/
    FLASH->NSCR |= FLASH_SECCR_SECSTRT;

    /*wait until the operation ends*/
    while (FLASH->NSSR & FLASH_SECSR_SECBSY);

    /*check for error*/
    if ((FLASH->NSSR & FLASH_SECSR_SECOPERR))
    {
    return (1);
    }
    /*reset CR*/
    FLASH->NSCR &= (~FLASH_SECCR_SECPER);
    }
  }

  else            // Flash secure
  {
    if (GetFlashBankMode() == 1U)  // Flash secure DUAL BANK
      {
    /*page erase enabled*/
    FLASH->SECCR |= FLASH_SECCR_SECPER;

    if(adr >= bankADD) // bank 2
        {
        page = (adr >>13);
        page = page - 0x80;
        page = page & 0x0000007F;
         /*Bank 2 Erase Selection*/
        FLASH->SECCR |= (FLASH_BKER);
         }
    else        // bank 1
        {
         page = ((adr >>13) & 0x0000007F );
         /*Bank 1 Erase Selection*/
         FLASH->SECCR &= ~(FLASH_BKER);
         }
       }
      else{                      // Flash non-secure SINGLE BANK
        /*Calculate the page */
              page = ((adr >>14) & 0x000000FF );
         }
    /*set PNB*/
    FLASH->SECCR &= ~(FLASH_SECCR_SECPNB);
    FLASH->SECCR |= page << 3;

    /*Start the erase operation*/
    FLASH->SECCR |= FLASH_SECCR_SECSTRT;

    /*wait until the operation ends*/
    while (FLASH->SECSR & FLASH_SECSR_SECBSY);

    /*check for error*/
  if ((FLASH->SECSR & FLASH_SECSR_SECOPERR))
    {
    return (1);
    }
    /*reset CR*/
  FLASH->SECCR &= (~FLASH_SECCR_SECPER);
  }
  #else
  bankADD=0x08100000;

  if (GetFlashSecureMode() == 0U) {   // Flash non-secure
    {

    /*page erase enabled*/
    FLASH->NSCR |= FLASH_SECCR_SECPER;

    if(adr >= bankADD) // bank 2
        {
        page = (adr >>13);
        page = page - 0x80;
        page = page & 0x0000007F;
        /*Bank 2 Erase Selection*/
        FLASH->NSCR |= (FLASH_BKER);
        }
    else               // bank 1
        {
         page = ((adr >>13) & 0x0000007F );
        /*Bank 1 Erase Selection*/
         FLASH->NSCR &= ~(FLASH_BKER);
        }

    /*set PNB*/
    FLASH->NSCR &= ~(FLASH_SECCR_SECPNB);
    FLASH->NSCR |= page << 3;

    /*Start erase operation*/
    FLASH->NSCR |= FLASH_SECCR_SECSTRT;

    /*wait until the operation ends*/
    while (FLASH->NSSR & FLASH_SECSR_SECBSY);

    /*check for error*/
    if ((FLASH->NSSR & FLASH_SECSR_SECOPERR))
    {
    return (1);
    }
    /*reset CR*/
    FLASH->NSCR &= (~FLASH_SECCR_SECPER);
    }
  }

  else            // Flash secure
  {
    /*page erase enabled*/
    FLASH->SECCR |= FLASH_SECCR_SECPER;

    if(adr >= bankADD) // bank 2
        {
        page = (adr >>13);
        page = page - 0x80;
        page = page & 0x0000007F;
         /*Bank 2 Erase Selection*/
        FLASH->SECCR |= (FLASH_BKER);
         }
    else        // bank 1
        {
         page = ((adr >>13) & 0x0000007F );
         /*Bank 1 Erase Selection*/
         FLASH->SECCR &= ~(FLASH_BKER);
         }

    /*set PNB*/
    FLASH->SECCR &= ~(FLASH_SECCR_SECPNB);
    FLASH->SECCR |= page << 3;

    /*Start the erase operation*/
    FLASH->SECCR |= FLASH_SECCR_SECSTRT;

    /*wait until the operation ends*/
    while (FLASH->SECSR & FLASH_SECSR_SECBSY);

    /*check for error*/
  if ((FLASH->SECSR & FLASH_SECSR_SECOPERR))
    {
    return (1);
    }
    /*reset CR*/
  FLASH->SECCR &= (~FLASH_SECCR_SECPER);
  }
  #endif

  return (0);
}




#if defined FLASH_OPT || defined FLASH_OTP
int EraseSector (unsigned long adr) {
  /* erase sector is not needed for
     - Flash Option bytes
     - Flash One Time Programmable bytes
  */
  return (0);
}
#endif /* FLASH_OPT || defined FLASH_OTP */


/*
 *  Program Page in Flash Memory
 *    Parameter:      adr:  Page Start Address
 *                    sz:   Page Size
 *                    buf:  Page Data
 *    Return Value:   0 - OK,  1 - Failed
 */


int ProgramPage (unsigned long adr, unsigned long sz, unsigned char *buf) {
   unsigned char tab[8];
   int i;
  unsigned long align = adr%8;
  unsigned long tempadd = 0;
  sz = (sz + 15) & ~15;                                    /* Adjust size for two Doublewords */

  FLASH->NSSR  = FLASH_PGERR;                              /* Reset Error Flags */
 if (GetFlashSecureMode() == 0U)        // Flash non-secure
 {
  FLASH->NSCR = FLASH_PG ;                               /* Programming Enabled */

  while (sz) {
         if(sz>=8)
      {
        FLASH->NSSR  = 0x000001FF;
          if (align !=0)
      {
         tempadd = (adr -align);
            for(i=0;i<align;i++)
        {
          tab[i]= *((unsigned char *)tempadd);
          tempadd=tempadd+1;
        }
          for(i=0;i<8-align;i++)
        {
          tab[i+align]=*((unsigned char *)buf);
          buf=buf+1;
        }
             tempadd = (adr -align);
      while (FLASH->NSSR  & FLASH_SECSR_SECBSY );
   /*write double world*/
    M32(tempadd ) = *((u32 *)tab );                   // Program the first word of the Double Word
    M32(tempadd + 4) = *((u32 *)(tab + 4));
        __asm volatile("NOP");
        __asm volatile("NOP");

               sz =sz -(8-align);
               adr= adr +(8-align);
               align = 0;
  }
      while (FLASH->NSSR & FLASH_SECSR_SECBSY );
   /*write double world*/
      M32(adr ) = *((u32 *)buf );                   // Program the first word of the Double Word
      M32(adr + 4) = *((u32 *)(buf + 4));
        buf +=8;
        sz -= 8;
        adr +=8;
        __asm volatile("NOP");
        __asm volatile("NOP");
      }
      else{
         /*write a word completed with 0xFF*/

        for(i=0;i<sz;i++)
        {
          tab[i]= *((unsigned char *)buf);
          buf=buf+1;
        }
         for(i=0;i<8-sz;i++)
        {
          tab[i+sz]=0xFF;
        }
      while (FLASH->NSSR & FLASH_SECSR_SECBSY){};


        M32(adr ) = *((u32 *)tab );                   // Program the first word of the Double Word
        M32(adr + 4) = *((u32 *)(tab + 4));

         sz =0;
      }
        /*wait until the operation ends*/
      while (FLASH->NSSR & FLASH_SECSR_SECBSY );
      if (FLASH->NSSR & FLASH_PGERR) {                       /* Check for Error */
      FLASH->NSSR  = FLASH_PGERR;                          /* Reset Error Flags */
      return (1);                                        /* Failed */
    }

  }

  FLASH->NSCR = 0U;                                        /* Reset CR */
}
 else {                    // Flash secure

  FLASH->SECCR = FLASH_PG ;                               /* Programming Enabled */

  while (sz) {

       if(sz>=8)
      {
        FLASH->SECSR  = 0x000001FF;
          if (align !=0)
      {
         tempadd = (adr -align);
            for(i=0;i<align;i++)
        {
          tab[i]= *((unsigned char *)tempadd);
          tempadd=tempadd+1;
        }
          for(i=0;i<8-align;i++)
        {
          tab[i+align]=*((unsigned char *)buf);
          buf=buf+1;
        }
             tempadd = (adr -align);
      while (FLASH->SECSR  & FLASH_SECSR_SECBSY );
   /*write double world*/
    M32(tempadd ) = *((u32 *)tab );                   // Program the first word of the Double Word
    M32(tempadd + 4) = *((u32 *)(tab + 4));
        __asm volatile("NOP");
        __asm volatile("NOP");

               sz =sz -(8-align);
               adr= adr +(8-align);
               align = 0;
  }
      while (FLASH->SECSR & FLASH_SECSR_SECBSY );
   /*write double world*/
      M32(adr ) = *((u32 *)buf );                   // Program the first word of the Double Word
      M32(adr + 4) = *((u32 *)(buf + 4));
        buf +=8;
        sz -= 8;
        adr +=8;
        __asm volatile("NOP");
        __asm volatile("NOP");
      }
      else{
         /*write a word completed with 0xFF*/

        for(i=0;i<sz;i++)
        {
          tab[i]= *((unsigned char *)buf);
          buf=buf+1;
        }
         for(i=0;i<8-sz;i++)
        {
          tab[i+sz]=0xFF;
        }
      while (FLASH->SECSR & FLASH_SECSR_SECBSY){};


        M32(adr ) = *((u32 *)tab );                   // Program the first word of the Double Word
        M32(adr + 4) = *((u32 *)(tab + 4));

         sz =0;
      }
        /*wait until the operation ends*/
      while (FLASH->SECSR & FLASH_SECSR_SECBSY );
      if (FLASH->SECSR & FLASH_PGERR) {                       /* Check for Error */
      FLASH->SECSR  = FLASH_PGERR;                          /* Reset Error Flags */
      return (1);                                        /* Failed */
    }


  }

  FLASH->SECCR = 0U;                                        /* Reset CR */
 }
  return (0);
}

